%% Load the data
clear all;

data = load('housing.data');

% split into predictors X and response variable Y
X = data(:, 1:end-1);
Y = data(:,end);

%% repeatedly run cross validation

% We will test all the numbers of subsets in this array
numSubsets = 2:20;

m = 100;

tic

% Rather than looping over the values directly, we loop a counter so that
% we can save the results easily 
for n = 1:numel(numSubsets)

    % For each value of n, we run cross validation m times.
    for k = 1:m

        % Generate the random subsets
        subsets = genRandomSubsets(X, numSubsets(n));

        % Run the linear regression model and get errors. Note: this
        % returns a STRUCT, one of the easiest ways of keeping track of a
        % set of related variables.
        profile on; [model] = runCrossVal(X, Y, subsets); profile viewer;

        % Record the errors and the model weights
        w{n,k} = model.w;
        trainMAE(n,k) = mean(model.trainMAE);
        testMAE(n,k) = mean(model.testMAE);

    end
end

toc

%% plot the estimates of train and test error

figure();

% ERRORBAR lets us plot with errorbars to see the variance in our random
% procedure. We can compute along the COLUMNS by passing in a '2' parameter
% to the mean and std commands for mean and standard error computations.

errorbar(numSubsets, mean(trainMAE,2), std(trainMAE, 0, 2)/sqrt(m), '-xb');

hold on;
errorbar(numSubsets, mean(testMAE,2), std(testMAE, 0, 2)/sqrt(m), '-or');
hold off;

% Making it easier to read
xlabel('Number of Subsets');
ylabel('Mean Absolute Error (MAE)');
xlim([0 20]);
title('Estimating performance with different numbers of folds');
legend('Train MAE', 'Test MAE');

%% plot the model weights

% Let's take the mean of the parameter vectors found when splitting the
% data in half.

% If we use curly braces to index into a cell array, that returns each cell
% as a *separate* result. In other words, calling w{1,:} will return 100
% different 13x1 vectors. To concatenate them, we have to capture that
% output with a concatenation operator.

figure();
wSplits = [w{1,:}]; % horizontally concatenate
wSplits = mean(wSplits,2);
bar(wSplits);
xlabel('Predictor number');
ylabel('Weight');

